@@ -1,6 +1,6 @@
 /* io_put.c
  *
- * Copyright (c) 1996-2006 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2008 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -10,6 +10,8 @@
 #	pragma hdrstop
 #endif
 
+#define _CRT_SECURE_NO_WARNINGS 1
+
 #ifndef O_BINARY
 	/* Needed for platforms using different EOLN sequence (i.e. DOS) */
 #	ifdef _O_BINARY
@@ -169,6 +171,7 @@ FTPPutOneF(
 	int sameAsRemote;
 	int skiprc;
 	size_t skipbufsize;
+	longstring cmdStr;
 	
 	if (cip->buf == NULL) {
 		FTPLogError(cip, kDoPerror, "Transfer buffer not allocated.\n");
@@ -241,8 +244,8 @@ FTPPutOneF(
 		if (appendflag == kAppendYes) {
 			zaction = kConfirmResumeProcSaidAppend;
 		} else if (
-				(cip->hasREST == kCommandNotAvailable) ||
-		/*		(xtype != kTypeBinary) ||		*/
+		/*		(cip->hasREST == kCommandNotAvailable) || We can now try to use APPE when REST is not availble. */
+		/*		(xtype != kTypeBinary) || We can now resume in ASCII too. */
 				(fstatrc < 0)
 		) {
 			zaction = kConfirmResumeProcSaidOverwrite;
@@ -378,25 +381,28 @@ FTPPutOneF(
 
 	vzaction = zaction;
 	if (vzaction == kConfirmResumeProcSaidAppend) {
-		cmd = "APPE";
+		cmd = "APPE ";
 		tmppfx = "";	/* Can't use that here. */
 		tmpsfx = "";
 	} else {
-		cmd = "STOR";
+		cmd = "STOR ";
 		if (tmppfx == NULL)
 			tmppfx = "";
 		if (tmpsfx == NULL)
 			tmpsfx = "";
 	}
 
-	tmpResult = FTPStartDataCmd(
+	memcpy(cmdStr, cmd, 4 + 1 /* space */ + 1 /* nul byte */);
+	STRNCAT(cmdStr, dstfile);
+
+	tmpResult = FTPStartDataCmd2(
 		cip,
 		kNetWriting,
 		xtype,
 		startPoint,
-		"%s %s",
-		cmd,
-		dstfile
+		cmdStr,
+		sizeof(cmdStr),
+		"(not used)"
 	);
 
 	if (tmpResult < 0) {
