@@ -1,6 +1,6 @@
 /* open.c
  *
- * Copyright (c) 1996-2004 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2005 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -10,6 +10,43 @@
 #	pragma hdrstop
 #endif
 
+void
+FTPResetStatusVariables(const FTPCIPtr cip)
+{
+	cip->errNo = 0;
+	memset(&cip->lastFTPCmdResultStr, 0, sizeof(cip->lastFTPCmdResultStr));
+	cip->lastFTPCmdResultNum = 0;
+	cip->sec = 0;
+	cip->secLeft = 0;
+	cip->kBytesPerSec = 0;
+	cip->percentCompleted = 0;
+	cip->expectedSize = 0;
+	cip->mdtm = 0;
+	cip->nextProgressUpdate = 0;
+	cip->stalled = 0;
+	cip->dataTimedOut = 0;
+	cip->cancelXfer = 0;
+	cip->canceling = 0;
+	cip->canceled = 0;
+	cip->connected = 0;
+	cip->loggedIn = 0;
+	cip->bytesTransferred = 0;
+	cip->curTransferType = 0;
+	cip->startPoint = 0;
+	cip->dataSocketConnected = 0;
+	cip->totalDials = 0;
+	memset(&cip->connectTime, 0, sizeof(cip->connectTime));
+	memset(&cip->loginTime, 0, sizeof(cip->loginTime));
+	memset(&cip->disconnectTime, 0, sizeof(cip->disconnectTime));
+	memset(&cip->lastCmdStart, 0, sizeof(cip->lastCmdStart));
+	memset(&cip->lastCmdFinish, 0, sizeof(cip->lastCmdFinish));
+	cip->numDownloads = 0;
+	cip->numUploads = 0;
+	cip->numListings = 0;
+}	/* FTPResetStatusVariables */
+
+
+
 
 void
 FTPDeallocateHost(const FTPCIPtr cip)
@@ -35,6 +72,7 @@ FTPDeallocateHost(const FTPCIPtr cip)
 	DisposeSReadlineInfo(&cip->ctrlSrl);
 #endif
 	DisposeLineListContents(&cip->lastFTPCmdResultLL);
+
 }	/* FTPDeallocateHost */
 
 
@@ -114,7 +152,7 @@ FTPLoginHost(const FTPCIPtr cip)
 		/* Try to get the email address if you didn't specify
 		 * a password when the user is anonymous.
 		 */
-		if (cip->pass[0] == '\0') {
+		if ((cip->pass[0] == '\0') && (cip->passIsEmpty == 0)) {
 			FTPInitializeAnonPassword(cip->lip);
 			(void) STRNCPY(cip->pass, cip->lip->defaultAnonPassword);
 		}
@@ -210,7 +248,7 @@ FTPLoginHost(const FTPCIPtr cip)
 				
 			case 331:	/* 331 User name okay, need password. */
 				if ((cip->firewallType == kFirewallNotInUse) || (fwloggedin != 0) || (fwsentpass != 0)) {
-					if ((cip->pass[0] == '\0') && (cip->passphraseProc != kNoFTPGetPassphraseProc))
+					if ((cip->pass[0] == '\0') && (cip->passIsEmpty == 0) && (cip->passphraseProc != kNoFTPGetPassphraseProc))
 						(*cip->passphraseProc)(cip, &rp->msg, cip->pass, sizeof(cip->pass));
 					ReInitResponse(cip, rp);
 					result = RCmd(cip, rp, "PASS %s", cip->pass);
@@ -274,7 +312,7 @@ okay:
 	if (cip->onLoginMsgProc != 0)
 		(*cip->onLoginMsgProc)(cip, rp);
 	DoneWithResponse(cip, rp);
-	result = 0;
+	result = kNoErr;
 	cip->loggedIn = 1;
 
 	/* Make a note of what our root directory is.
@@ -299,13 +337,10 @@ okay:
 	 * are logged in and do not need it any more.
 	 */
 	if ((anonLogin == 0) && (cip->leavePass == 0))
-		(void) memset(cip->pass, '*', strlen(cip->pass));
+		(void) memset(cip->pass, '*', sizeof(cip->pass) - 1);
 
-	if (result < 0)
-		cip->errNo = result;
-	else
-		(void) gettimeofday(&cip->loginTime, NULL);
-	return result;
+	(void) gettimeofday(&cip->loginTime, NULL);
+	return result;	/* kNoErr */
 
 done:
 	DoneWithResponse(cip, rp);
@@ -322,13 +357,17 @@ done2:
 				/* Don't leave cleartext password in memory,
 				 * since we won't be redialing.
 				 */
-				(void) memset(cip->pass, '*', strlen(cip->pass));
+				(void) memset(cip->pass, '*', sizeof(cip->pass) - 1);
 		}
 	}
-	if (result < 0)
+	if (result > 0) {
+		result = cip->errNo = kErrLoginHostMiscErr;
+	}
+	if (result < 0) {
 		cip->errNo = result;
-	else
+	} else {
 		(void) gettimeofday(&cip->loginTime, NULL);
+	}
 	return result;
 }	/* FTPLoginHost */
 
@@ -400,6 +439,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 		cip->hasPASV = kCommandAvailable;
 		cip->hasSIZE = kCommandNotAvailable;
 		cip->hasMDTM = kCommandNotAvailable;
+		cip->hasMDTM_set = kCommandNotAvailable;
 		cip->hasREST = kCommandNotAvailable;
 		cip->NLSTfileParamWorks = kCommandAvailable;
 		cip->hasCLNT = kCommandNotAvailable;
@@ -447,6 +487,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 				cip->hasPASV = kCommandAvailable;
 				cip->hasSIZE = kCommandAvailable;
 				cip->hasMDTM = kCommandAvailable;
+				cip->hasMDTM_set = kCommandAvailable;
 				cip->hasREST = kCommandAvailable;
 				cip->NLSTfileParamWorks = kCommandAvailable;
 			} else if (cip->serverType == kServerTypeNcFTPd) {
@@ -475,7 +516,9 @@ FTPQueryFeatures(const FTPCIPtr cip)
 				 * (c) A protocol violation.
 				 */
 				cp = lp->line;
-				if (*cp++ != ' ')
+				while ((*cp != '\0') && (isspace(*cp)))
+					cp++;
+				if (*cp == '\0')
 					continue;
 				if (ISTRNCMP(cp, "PASV", 4) == 0) {
 					cip->hasPASV = kCommandAvailable;
@@ -682,6 +725,49 @@ FTPInitialLogEntry(const FTPCIPtr cip)
 		}
 #endif
 
+#ifdef LINUX
+	{
+		FILE *etc_fp;
+		const char **etc_fn;
+		char etc_line[256], *etc_cp;
+
+		const char *etc_fnames[] = {
+			"/etc/yellowdog-release",
+			"/etc/gentoo-release",
+			"/etc/turbolinux-release",
+			"/etc/slackware-release",
+			"/etc/slackware-version",
+			"/etc/mandrake-release",
+			"/etc/debian_version",
+			"/etc/conectiva-release",
+			"/etc/SuSE-release",
+			"/etc/fedora-release",
+			"/etc/redhat-release",	/* Works for CentOS, too. */
+			"/etc/issue",
+			NULL,
+		};
+
+		for (etc_fn = etc_fnames; *etc_fn != NULL; etc_fn++) {
+			etc_fp = fopen(*etc_fn, "r");
+			if (etc_fp != NULL) {
+				PrintF(cip, "Contents of %.127s:\n", *etc_fn);
+				memset(etc_line, 0, sizeof(etc_line));
+				while (fgets(etc_line, sizeof(etc_line) - 1, etc_fp) != NULL) {
+					etc_cp = etc_line + strlen(etc_line) - 1;
+					while ((etc_cp >= etc_line) && (isspace((int) *etc_cp)))
+						etc_cp--;
+					++etc_cp;
+					*etc_cp = '\0';
+					if (etc_line[0] != '\0') {
+						PrintF(cip, "  %.127s\n", etc_line);
+					}
+				}
+				(void) fclose(etc_fp);
+			}
+		}
+	}
+#endif	/* LINUX */
+
 #if defined(HAVE_GNU_GET_LIBC_VERSION) && defined(HAVE_GNU_GET_LIBC_RELEASE)
 		PrintF(cip, "Glibc: %s (%s)\n",
 			gnu_get_libc_version(),
@@ -775,6 +861,7 @@ FTPOpenHost(const FTPCIPtr cip)
 		return (kErrNoHostSpecified);
 	}
 
+	FTPResetStatusVariables(cip);
 	FTPManualOverrideFeatures(cip);
 	FTPInitialLogEntry(cip);
 
@@ -840,6 +927,9 @@ FTPOpenHost(const FTPCIPtr cip)
 			 * from OpenControlConnection().
 			 */
 			PrintF(cip, "Cannot recover from miscellaneous open error %d.\n", result);
+			if (result > 0)
+				result = kErrOpenHostMiscErr;
+			FTPDeallocateHost(cip);
 			return result;
 		}
 
@@ -858,6 +948,10 @@ FTPOpenHost(const FTPCIPtr cip)
 			}
 		}
 	}
+	if (result > 0)
+		result = kErrOpenHostMiscErr;
+	if (result != kNoErr)
+		FTPDeallocateHost(cip);
 	return (result);
 }	/* FTPOpenHost */
 
@@ -899,11 +993,13 @@ FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf,
 	cip->ctrlSocketR = kClosedFileDescriptor;
 	cip->ctrlSocketW = kClosedFileDescriptor;
 	cip->dataPortMode = kDefaultDataPortMode;
+	cip->maxNumberOfSuccessivePASVAttempts = kDefaultMaxNumberOfSuccessivePASVAttempts;
 	cip->dataSocket = kClosedFileDescriptor;
 	cip->lip = lip;
 	cip->hasPASV = kCommandAvailabilityUnknown;
 	cip->hasSIZE = kCommandAvailabilityUnknown;
 	cip->hasMDTM = kCommandAvailabilityUnknown;
+	cip->hasMDTM_set = kCommandAvailabilityUnknown;
 	cip->hasREST = kCommandAvailabilityUnknown;
 	cip->hasNLST_a = kCommandAvailabilityUnknown;
 	cip->hasNLST_d = kCommandAvailabilityUnknown;
@@ -924,7 +1020,8 @@ FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf,
 	cip->firewallType = kFirewallNotInUse;
 	cip->startingWorkingDirectory = NULL;
 	cip->shutdownUnusedSideOfSockets = 0;
-
+	cip->asciiTranslationMode = kAsciiTranslationModeDefault;
+	
 #ifdef MACOSX
 	/* For Mac OS 9 compatibility you could set this to '\r' */
 	cip->textEOLN[0] = '\n';
