@@ -263,6 +263,7 @@ main(int argc, char **argv)
 			break;
 		case 'p':
 			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			fi.leavePass = 1;
 			if (fi.pass[0] == '\0')
 				fi.passIsEmpty = 1;
 			memset(opt.arg, 0, strlen(fi.pass));
@@ -372,6 +373,11 @@ main(int argc, char **argv)
 	if (progmeters != 0)
 		fi.progress = PrStatBar;
 
+	if ((rflag != 0) && (ftpcat != 0)) {
+		fprintf(stderr, "The -R flag is not supported with -c/-C.\n\n");
+		Usage();
+	}
+
 	if ((perfilecmd[0] != '\0') && (rflag != 0))
 		(void) fprintf(stderr, "Warning: your -X command is only applied once per command-line parameter, and not for each file in the directory.\n");
 
@@ -617,12 +623,18 @@ main(int argc, char **argv)
 				es = kExitXferFailed;
 			}
 		} else if (ftpcat != 0) {
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+			_setmode(STDOUT_FILENO, _O_BINARY);
+#endif
 			if (FTPGetOneFile3(&fi, urlfile, NULL, xtype, STDOUT_FILENO, resumeflag, kAppendNo, deleteflag, kNoFTPConfirmResumeDownloadProc, 0) == kNoErr) {
 				es = kExitSuccess;
 			} else {
 				FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
 				es = kExitXferFailed;
 			}
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+			_setmode(STDOUT_FILENO, _O_TEXT);
+#endif
 		} else {
 			if ((rc = FTPGetFiles3(&fi, urlfile, ".", rflag, kGlobYes, xtype, resumeflag, appendflag, deleteflag, tarflag, kNoFTPConfirmResumeDownloadProc, 0)) < 0) {
 				if (rc == kErrLocalSameAsRemote) {
@@ -648,10 +660,16 @@ main(int argc, char **argv)
 			else
 				es = kExitXferFailed;
 		} else if (ftpcat != 0) {
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+			_setmode(STDOUT_FILENO, _O_BINARY);
+#endif
 			if (FTPGetOneFile3(&fi, flist[0], NULL, xtype, STDOUT_FILENO, resumeflag, kAppendNo, deleteflag, kNoFTPConfirmResumeDownloadProc, 0) == kNoErr)
 				es = kExitSuccess;
 			else
 				es = kExitXferFailed;
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+			_setmode(STDOUT_FILENO, _O_TEXT);
+#endif
 		} else {
 			if (Copy(&fi, dstdir, flist, rflag, xtype, resumeflag, appendflag, deleteflag, tarflag, perfilecmd) == 0)
 				es = kExitSuccess;
