@@ -5,8 +5,9 @@
  * 
  */
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	pragma once
+#	define _CRT_SECURE_NO_WARNINGS 1
 #	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
 #	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
 #	pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
@@ -58,8 +59,12 @@
 #	include <fcntl.h>
 #	include <signal.h>
 #	include <assert.h>
-#	define strcasecmp stricmp
-#	define strncasecmp strnicmp
+#	ifndef strcasecmp
+#		define strcasecmp _stricmp
+#		define strncasecmp _strnicmp
+#		define strdup _strdup
+#		define fdopen _fdopen
+#	endif
 #	define sleep WinSleep
 #	ifndef S_ISREG
 #		define S_ISREG(m)      (((m) & _S_IFMT) == _S_IFREG)
@@ -227,7 +232,7 @@
 
 #define NDEBUG 1			/* For assertions. */
 
-#if defined(HAVE_LONG_LONG) && defined(HAVE_OPEN64)
+#if ((defined(HAVE_LONG_LONG)) && (defined(_LARGEFILE64_SOURCE)) && (defined(HAVE_OPEN64)))
 #	define Open open64
 #else
 #	define Open open
@@ -237,7 +242,7 @@
 #	define Stat WinStat64
 #	define Lstat WinStat64
 #	define Fstat WinFStat64
-#elif defined(HAVE_LONG_LONG) && defined(HAVE_STAT64) && defined(HAVE_STRUCT_STAT64)
+#elif ((defined(HAVE_LONG_LONG)) && (defined(_LARGEFILE64_SOURCE)) && (defined(HAVE_STAT64)) && (defined(HAVE_STRUCT_STAT64)))
 #	define Stat stat64
 #	ifdef HAVE_FSTAT64
 #		define Fstat fstat64
@@ -255,18 +260,10 @@
 #	define Lstat lstat
 #endif
 
-#if defined(HAVE_LONG_LONG) && defined(HAVE_LSEEK64)
+#if ((defined(_FILE_OFFSET_BITS)) && (_FILE_OFFSET_BITS > 32))
+#	define Lseek(a,b,c) lseek(a, (off_t) b, c)
+#elif ((defined(HAVE_LONG_LONG)) && (defined(_LARGEFILE64_SOURCE)) && (defined(HAVE_LSEEK64)))
 #	define Lseek(a,b,c) lseek64(a, (longest_int) b, c)
-#elif defined(HAVE_LONG_LONG) && defined(HAVE_LLSEEK)
-#	if 1
-#		if defined(LINUX) && (LINUX <= 23000)
-#			define Lseek(a,b,c) lseek(a, (off_t) b, c)
-#		else
-#			define Lseek(a,b,c) llseek(a, (longest_int) b, c)
-#		endif
-#	else
-#		define Lseek(a,b,c) lseek(a, (off_t) b, c)
-#	endif
 #else
 #	define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #endif
